import pytest

from napari_allencell_annotator._style import Style


# class TestStyle:
# @pytest.mark.parametrize("name", [None, "not_a_qt_stylesheet.txt"])
# def test_load_stylesheet_bad_name(self, name):
#     with pytest.raises(ValueError):
#         Style.get_stylesheet(name)
#
# def test_load_stylesheet_bad_path(self):
#     with pytest.raises(IOError):
#         Style.get_stylesheet("not_found.qss")
#
# def test_load_stylesheet(self):
#     contents = Style.get_stylesheet("main.qss")
#     assert contents is not None
#     assert len(contents) > 0



from unittest import mock
from unittest.mock import MagicMock, create_autospec, ANY

from napari_allencell_annotator.view.annotator_view import (
    AnnotatorView,
    AnnotatorViewMode,
    QListWidget,
    QLineEdit,
    QSpinBox,
    QCheckBox,
    QComboBox,
    QVBoxLayout,
    QListWidgetItem,
    QWidget,
    QHBoxLayout,
    QLabel,
    QMessageBox,
)


class TestAnnotatorView:
    def setup_method(self):
        with mock.patch.object(AnnotatorView, "__init__", lambda x: None):
            self._view = AnnotatorView()
            self._view._mode = AnnotatorViewMode.ADD

    def test_mode(self):
        assert self._view.mode == AnnotatorViewMode.ADD

    def test_set_mode(self):
        expected = AnnotatorViewMode.VIEW
        self._view._display_mode = MagicMock()
        self._view.set_mode(expected)
        assert self._view._mode == self._view.mode == expected
        self._view._display_mode.assert_called_once_with()

    def test_set_num_images(self):
        self._view.set_num_images(4)
        assert self._view.num_images == 4

    def test_set_curr_index(self):
        self._view.num_images = 4
        self._view.progress_bar = MagicMock()
        self._view.progress_bar.setText = MagicMock()
        self._view.set_curr_index(2)
        self._view.progress_bar.setText.assert_called_once_with("3 of 4 Images")
        assert self._view.curr_index == 2

    def test_reset_annotations(self):
        with mock.patch.object(QListWidget, "__init__", lambda x: None):

            self._view.annot_list = QListWidget()

            self._view.annot_list.setMaximumHeight = MagicMock()
            self._view.annot_list.clear = MagicMock()
            self._view.annotation_item_widgets = ["item"]
            self._view.annots_order = ["item"]
            self._view.default_vals = ["item"]
            self._view._reset_annotations()

            self._view.annot_list.clear.assert_called_once_with()
            assert self._view.annotation_item_widgets == []
            assert self._view.annots_order == []
            assert self._view.default_vals == []

    def test_render_default_values(self):
        self._view.default_vals = []
        self._view.render_values = MagicMock()
        self._view.render_default_values()
        self._view.render_values.assert_called_once_with([])

    def test_render_values_line_input(self):
        q_line_edit = create_autospec(QLineEdit)
        self._view.annotation_item_widgets = [q_line_edit]
        self._view.default_vals = ["default"]
        self._view.render_values(["hello"])
        q_line_edit.setText.assert_called_once_with("hello")

    def test_render_values_line_no_input(self):
        q_line_edit = create_autospec(QLineEdit)
        self._view.annotation_item_widgets = [q_line_edit]
        self._view.default_vals = ["default"]
        self._view.render_values([""])
        q_line_edit.setText.assert_called_once_with("default")

    def test_render_values_line_none_input(self):
        q_line_edit = create_autospec(QLineEdit)
        self._view.annotation_item_widgets = [q_line_edit]
        self._view.default_vals = ["default"]
        self._view.render_values([None])
        q_line_edit.setText.assert_called_once_with("default")

    def test_render_values_spin(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        self._view.annotation_item_widgets = [q_spin]
        self._view.default_vals = [8]
        self._view.render_values(["0"])
        q_spin.setValue.assert_called_once_with(0)

    def test_render_values_spin_empty(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        self._view.annotation_item_widgets = [q_spin]
        self._view.default_vals = ["8"]
        self._view.render_values([""])
        q_spin.setValue.assert_called_once_with(8)

    def test_render_values_check(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        self._view.annotation_item_widgets = [q_check]
        self._view.default_vals = [True]
        self._view.render_values(["False"])
        q_check.setChecked.assert_called_once_with(False)

    def test_render_values_check_empty(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        self._view.annotation_item_widgets = [q_check]
        self._view.default_vals = [True]
        self._view.render_values([""])
        q_check.setChecked.assert_called_once_with(True)

    def test_render_values_combo(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        q_combo = create_autospec(QComboBox)
        self._view.annotation_item_widgets = [q_combo]
        self._view.default_vals = ["option1"]
        self._view.render_values(["option2"])
        q_combo.setCurrentText.assert_called_once_with("option2")

    def test_render_values_combo_empty(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        q_combo = create_autospec(QComboBox)
        self._view.annotation_item_widgets = [q_combo]
        self._view.default_vals = ["option1"]
        self._view.render_values([""])
        q_combo.setCurrentText.assert_called_once_with("option1")

    def test_render_values_mult_empty(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        q_combo = create_autospec(QComboBox)
        self._view.annotation_item_widgets = [q_line_edit, q_spin, q_check, q_combo]
        self._view.default_vals = ["default", "0", True, "option1"]
        self._view.render_values(["", "", "", ""])
        q_line_edit.setText.assert_called_once_with("default")
        q_spin.setValue.assert_called_once_with(0)
        q_check.setChecked.assert_called_once_with(True)
        q_combo.setCurrentText.assert_called_once_with("option1")

    def test_render_values_mult_some_empty(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        q_combo = create_autospec(QComboBox)
        self._view.annotation_item_widgets = [q_line_edit, q_spin, q_check, q_combo]
        self._view.default_vals = ["default", "0", True, "option1"]
        self._view.render_values(["", "5", "", "option2"])
        q_line_edit.setText.assert_called_once_with("default")
        q_spin.setValue.assert_called_once_with(5)
        q_check.setChecked.assert_called_once_with(True)
        q_combo.setCurrentText.assert_called_once_with("option2")

    def test_render_values_mult_some_none_some_empty(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        q_combo = create_autospec(QComboBox)
        self._view.annotation_item_widgets = [q_line_edit, q_spin, q_check, q_combo]
        self._view.default_vals = ["default", "0", True, "option1"]
        self._view.render_values([None, "", "False", ""])
        q_line_edit.setText.assert_called_once_with("default")
        q_spin.setValue.assert_called_once_with(0)
        q_check.setChecked.assert_called_once_with(False)
        q_combo.setCurrentText.assert_called_once_with("option1")

    def test_get_curr_annots(self):
        q_line_edit = create_autospec(QLineEdit)
        q_line_edit.text = MagicMock(return_value="text")
        q_spin = create_autospec(QSpinBox)
        q_spin.value = MagicMock(return_value=5)
        q_check = create_autospec(QCheckBox)
        q_check.isChecked = MagicMock(return_value=True)
        q_combo = create_autospec(QComboBox)
        q_combo.currentText = MagicMock(return_value="text2")
        self._view.annotation_item_widgets = [q_line_edit, q_spin, q_check, q_combo]
        assert self._view.get_curr_annots() == ["text", 5, True, "text2"]

    def test_toggle_annots_editable_true(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        q_combo = create_autospec(QComboBox)
        self._view.annotation_item_widgets = [q_line_edit, q_spin, q_check, q_combo]
        self._view.toggle_annots_editable(True)
        q_line_edit.setEnabled.assert_called_once_with(True)
        q_spin.setEnabled.assert_called_once_with(True)
        q_check.setEnabled.assert_called_once_with(True)
        q_combo.setEnabled.assert_called_once_with(True)

    def test_toggle_annots_editable_false(self):
        q_line_edit = create_autospec(QLineEdit)
        q_spin = create_autospec(QSpinBox)
        q_check = create_autospec(QCheckBox)
        q_combo = create_autospec(QComboBox)
        self._view.annotation_item_widgets = [q_line_edit, q_spin, q_check, q_combo]
        self._view.toggle_annots_editable(False)
        q_line_edit.setEnabled.assert_called_once_with(False)
        q_spin.setEnabled.assert_called_once_with(False)
        q_check.setEnabled.assert_called_once_with(False)
        q_combo.setEnabled.assert_called_once_with(False)

    def test_display_mode_add_mode(self):
        self._view._mode = AnnotatorViewMode.ADD
        self._view.add_widget = MagicMock()
        self._view._reset_annotations = MagicMock()
        self._view.layout = create_autospec(QVBoxLayout)
        self._view.layout.count = MagicMock(return_value=3)
        item = create_autospec(QListWidgetItem)
        widget = create_autospec(QLineEdit)
        item.widget = MagicMock(return_value=widget)
        self._view.layout.itemAt = MagicMock(return_value=item)

        self._view._display_mode()

        self._view.layout.count.assert_called_once_with()
        item.widget.assert_called_once_with()
        widget.hide.assert_called_once_with()
        self._view.layout.removeItem.assert_called_once_with(item)
        self._view.add_widget.show.assert_called_once_with()
        self._view._reset_annotations.assert_called_once_with()
        self._view.layout.addWidget.assert_called_once_with(self._view.add_widget)

    def test_display_mode_view_mode(self):
        self._view._mode = AnnotatorViewMode.VIEW
        self._view.save_json_btn = MagicMock()
        self._view.view_widget = MagicMock()
        self._view.layout = create_autospec(QVBoxLayout)
        self._view.layout.count = MagicMock(return_value=3)
        item = create_autospec(QListWidgetItem)
        widget = create_autospec(QLineEdit)
        item.widget = MagicMock(return_value=widget)
        self._view.layout.itemAt = MagicMock(return_value=item)

        self._view._display_mode()

        self._view.layout.count.assert_called_once_with()
        item.widget.assert_called_once_with()
        widget.hide.assert_called_once_with()
        self._view.layout.removeItem.assert_called_once_with(item)
        self._view.view_widget.show.assert_called_once_with()
        self._view.save_json_btn.setEnabled.assert_called_once_with(True)
        self._view.layout.addWidget.assert_called_once_with(self._view.view_widget)

    def test_display_mode_annotate_mode(self):
        self._view._mode = AnnotatorViewMode.ANNOTATE
        self._view.prev_btn = MagicMock()
        self._view.annot_widget = MagicMock()
        self._view.layout = create_autospec(QVBoxLayout)
        self._view.layout.count = MagicMock(return_value=3)
        item = create_autospec(QListWidgetItem)
        widget = create_autospec(QLineEdit)
        item.widget = MagicMock(return_value=widget)
        self._view.layout.itemAt = MagicMock(return_value=item)

        self._view._display_mode()

        self._view.layout.count.assert_called_once_with()
        item.widget.assert_called_once_with()
        widget.hide.assert_called_once_with()
        self._view.layout.removeItem.assert_called_once_with(item)
        self._view.annot_widget.show.assert_called_once_with()
        self._view.prev_btn.setEnabled.assert_called_once_with(False)
        self._view.layout.addWidget.assert_called_once_with(self._view.annot_widget)

    def test_render_annotations(self):
        dct = {"name": [], "name2": [], "name3": []}
        self._view._create_annot = MagicMock()

        self._view.annot_list = create_autospec(QListWidget)
        self._view.annot_list.item(0).sizeHint().height = MagicMock(return_value=10)
        self._view.render_annotations(dct)
        self._view._create_annot.assert_has_calls(
            [mock.call("name", []), mock.call("name2", []), mock.call("name3", [])]
        )
        self._view.annot_list.setMaximumHeight.assert_called_once_with(30)

    def test_render_annotations_one_item(self):
        dct = {"name": []}
        self._view._create_annot = MagicMock()

        self._view.annot_list = create_autospec(QListWidget)
        self._view.annot_list.item(0).sizeHint().height = MagicMock(return_value=10)
        self._view.render_annotations(dct)
        self._view._create_annot.assert_has_calls([mock.call("name", [])])
        self._view.annot_list.setMaximumHeight.assert_called_once_with(10)

    def test_create_annot_string(self):
        with mock.patch.object(QWidget, "__init__", lambda x: None):
            with mock.patch.object(QHBoxLayout, "__init__", lambda x: None):
                with mock.patch.object(QLabel, "__init__", lambda x, y: None):
                    with mock.patch.object(QLineEdit, "__init__", lambda x, y: None):
                        with mock.patch.object(QListWidget, "__init__", lambda y: None):
                            with mock.patch.object(QListWidget, "__init__", lambda y: None):
                                with mock.patch.object(QListWidgetItem, "__init__", lambda x, y: None):
                                    QLineEdit.setEnabled = MagicMock()
                                    self._view.annots_order = []
                                    self._view.default_vals = []
                                    self._view.annotation_item_widgets = []
                                    QHBoxLayout.setContentsMargins = MagicMock()
                                    QHBoxLayout.setSpacing = MagicMock()
                                    QWidget.setLayout = MagicMock()
                                    QWidget.minimumSizeHint = MagicMock()
                                    QListWidgetItem.setSizeHint = MagicMock()
                                    self._view.annot_list = QListWidget()
                                    self._view.annot_list.setItemWidget = MagicMock()
                                    QHBoxLayout.addWidget = MagicMock()

                                    QHBoxLayout.setSizeConstraint = MagicMock()
                                    self._view._create_annot("name", {"type": "string", "default": ""})

                                    assert self._view.annots_order == ["name"]
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QLabel("name")))
                                    line_edit = QLineEdit("")
                                    assert QHBoxLayout().addWidget.has_call(mock.call(line_edit))
                                    assert self._view.default_vals == [""]
                                    assert len(self._view.annotation_item_widgets) == 1
                                    assert type(self._view.annotation_item_widgets[0]) == type(QLineEdit(""))
                                    QLineEdit("").setEnabled.assert_called_once_with(True)
                                    QHBoxLayout().setContentsMargins.assert_called_once_with(2, 12, 8, 12)
                                    QHBoxLayout().setSpacing.assert_called_once_with(2)
                                    QWidget().setLayout.assert_called_once_with(ANY)
                                    assert isinstance(QWidget().setLayout.call_args_list[0][0][0], QHBoxLayout)

                                    QListWidgetItem(self._view.annot_list).setSizeHint.assert_called_once_with(
                                        QWidget().minimumSizeHint()
                                    )

                                    self._view.annot_list.setItemWidget.assert_called_once_with(ANY, ANY)
                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][0], QListWidgetItem
                                    )

                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][1], QWidget
                                    )

    def test_create_annot_number(self):
        with mock.patch.object(QWidget, "__init__", lambda x: None):
            with mock.patch.object(QHBoxLayout, "__init__", lambda x: None):
                with mock.patch.object(QLabel, "__init__", lambda x, y: None):
                    with mock.patch.object(QSpinBox, "__init__", lambda y: None):
                        with mock.patch.object(QListWidget, "__init__", lambda y: None):
                            with mock.patch.object(QListWidget, "__init__", lambda y: None):
                                with mock.patch.object(QListWidgetItem, "__init__", lambda x, y: None):
                                    QSpinBox.setEnabled = MagicMock()
                                    QSpinBox.setValue = MagicMock()
                                    self._view.annots_order = []
                                    self._view.default_vals = []
                                    self._view.annotation_item_widgets = []
                                    QHBoxLayout.setContentsMargins = MagicMock()
                                    QHBoxLayout.setSpacing = MagicMock()
                                    QWidget.setLayout = MagicMock()
                                    QWidget.minimumSizeHint = MagicMock()
                                    QListWidgetItem.setSizeHint = MagicMock()
                                    self._view.annot_list = QListWidget()
                                    self._view.annot_list.setItemWidget = MagicMock()
                                    QHBoxLayout.addWidget = MagicMock()

                                    QHBoxLayout.setSizeConstraint = MagicMock()
                                    self._view._create_annot("name", {"type": "number", "default": 2})

                                    assert self._view.annots_order == ["name"]
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QLabel("name")))
                                    QSpinBox().setValue.assert_called_once_with(2)
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QSpinBox()))
                                    assert self._view.default_vals == [2]
                                    assert len(self._view.annotation_item_widgets) == 1
                                    assert type(self._view.annotation_item_widgets[0]) == type(QSpinBox())
                                    QSpinBox().setEnabled.assert_called_once_with(True)
                                    QHBoxLayout().setContentsMargins.assert_called_once_with(2, 12, 8, 12)
                                    QHBoxLayout().setSpacing.assert_called_once_with(2)
                                    QWidget().setLayout.assert_called_once_with(ANY)
                                    assert isinstance(QWidget().setLayout.call_args_list[0][0][0], QHBoxLayout)

                                    QListWidgetItem(self._view.annot_list).setSizeHint.assert_called_once_with(
                                        QWidget().minimumSizeHint()
                                    )

                                    self._view.annot_list.setItemWidget.assert_called_once_with(ANY, ANY)
                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][0], QListWidgetItem
                                    )

                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][1], QWidget
                                    )

    def test_create_annot_bool_true(self):
        with mock.patch.object(QWidget, "__init__", lambda x: None):
            with mock.patch.object(QHBoxLayout, "__init__", lambda x: None):
                with mock.patch.object(QLabel, "__init__", lambda x, y: None):
                    with mock.patch.object(QCheckBox, "__init__", lambda y: None):
                        with mock.patch.object(QListWidget, "__init__", lambda y: None):
                            with mock.patch.object(QListWidget, "__init__", lambda y: None):
                                with mock.patch.object(QListWidgetItem, "__init__", lambda x, y: None):
                                    QCheckBox.setEnabled = MagicMock()
                                    QCheckBox.setChecked = MagicMock()
                                    self._view.annots_order = []
                                    self._view.default_vals = []
                                    self._view.annotation_item_widgets = []
                                    QHBoxLayout.setContentsMargins = MagicMock()
                                    QHBoxLayout.setSpacing = MagicMock()
                                    QWidget.setLayout = MagicMock()
                                    QWidget.minimumSizeHint = MagicMock()
                                    QListWidgetItem.setSizeHint = MagicMock()
                                    self._view.annot_list = QListWidget()
                                    self._view.annot_list.setItemWidget = MagicMock()
                                    QHBoxLayout.addWidget = MagicMock()

                                    QHBoxLayout.setSizeConstraint = MagicMock()
                                    self._view._create_annot("name", {"type": "bool", "default": True})

                                    assert self._view.annots_order == ["name"]
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QLabel("name")))
                                    QCheckBox().setChecked.assert_called_once_with(True)
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QCheckBox()))
                                    assert self._view.default_vals == [True]
                                    assert len(self._view.annotation_item_widgets) == 1
                                    assert type(self._view.annotation_item_widgets[0]) == type(QCheckBox())
                                    QCheckBox().setEnabled.assert_called_once_with(True)
                                    QHBoxLayout().setContentsMargins.assert_called_once_with(2, 12, 8, 12)
                                    QHBoxLayout().setSpacing.assert_called_once_with(2)
                                    QWidget().setLayout.assert_called_once_with(ANY)
                                    assert isinstance(QWidget().setLayout.call_args_list[0][0][0], QHBoxLayout)

                                    QListWidgetItem(self._view.annot_list).setSizeHint.assert_called_once_with(
                                        QWidget().minimumSizeHint()
                                    )

                                    self._view.annot_list.setItemWidget.assert_called_once_with(ANY, ANY)
                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][0], QListWidgetItem
                                    )

                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][1], QWidget
                                    )

    def test_create_annot_bool_false(self):
        with mock.patch.object(QWidget, "__init__", lambda x: None):
            with mock.patch.object(QHBoxLayout, "__init__", lambda x: None):
                with mock.patch.object(QLabel, "__init__", lambda x, y: None):
                    with mock.patch.object(QCheckBox, "__init__", lambda y: None):
                        with mock.patch.object(QListWidget, "__init__", lambda y: None):
                            with mock.patch.object(QListWidget, "__init__", lambda y: None):
                                with mock.patch.object(QListWidgetItem, "__init__", lambda x, y: None):
                                    QCheckBox.setEnabled = MagicMock()
                                    QCheckBox.setChecked = MagicMock()
                                    self._view.annots_order = []
                                    self._view.default_vals = []
                                    self._view.annotation_item_widgets = []
                                    QHBoxLayout.setContentsMargins = MagicMock()
                                    QHBoxLayout.setSpacing = MagicMock()
                                    QWidget.setLayout = MagicMock()
                                    QWidget.minimumSizeHint = MagicMock()
                                    QListWidgetItem.setSizeHint = MagicMock()
                                    self._view.annot_list = QListWidget()
                                    self._view.annot_list.setItemWidget = MagicMock()
                                    QHBoxLayout.addWidget = MagicMock()

                                    QHBoxLayout.setSizeConstraint = MagicMock()
                                    self._view._create_annot("name", {"type": "bool", "default": False})

                                    assert self._view.annots_order == ["name"]
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QLabel("name")))
                                    QCheckBox().setChecked.assert_not_called()
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QCheckBox()))
                                    assert self._view.default_vals == [False]
                                    assert len(self._view.annotation_item_widgets) == 1
                                    assert type(self._view.annotation_item_widgets[0]) == type(QCheckBox())
                                    QCheckBox().setEnabled.assert_called_once_with(True)
                                    QHBoxLayout().setContentsMargins.assert_called_once_with(2, 12, 8, 12)
                                    QHBoxLayout().setSpacing.assert_called_once_with(2)
                                    QWidget().setLayout.assert_called_once_with(ANY)
                                    assert isinstance(QWidget().setLayout.call_args_list[0][0][0], QHBoxLayout)

                                    QListWidgetItem(self._view.annot_list).setSizeHint.assert_called_once_with(
                                        QWidget().minimumSizeHint()
                                    )

                                    self._view.annot_list.setItemWidget.assert_called_once_with(ANY, ANY)
                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][0], QListWidgetItem
                                    )

                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][1], QWidget
                                    )

    def test_create_annot_list(self):
        with mock.patch.object(QWidget, "__init__", lambda x: None):
            with mock.patch.object(QHBoxLayout, "__init__", lambda x: None):
                with mock.patch.object(QLabel, "__init__", lambda x, y: None):
                    with mock.patch.object(QComboBox, "__init__", lambda y: None):
                        with mock.patch.object(QListWidget, "__init__", lambda y: None):
                            with mock.patch.object(QListWidget, "__init__", lambda y: None):
                                with mock.patch.object(QListWidgetItem, "__init__", lambda x, y: None):
                                    QComboBox.setEnabled = MagicMock()
                                    QComboBox.addItem = MagicMock()
                                    QComboBox.setCurrentText = MagicMock()
                                    self._view.annots_order = []
                                    self._view.default_vals = []
                                    self._view.annotation_item_widgets = []
                                    QHBoxLayout.setContentsMargins = MagicMock()
                                    QHBoxLayout.setSpacing = MagicMock()
                                    QWidget.setLayout = MagicMock()
                                    QWidget.minimumSizeHint = MagicMock()
                                    QListWidgetItem.setSizeHint = MagicMock()
                                    self._view.annot_list = QListWidget()
                                    self._view.annot_list.setItemWidget = MagicMock()
                                    QHBoxLayout.addWidget = MagicMock()
                                    QHBoxLayout.setSizeConstraint = MagicMock()
                                    self._view._create_annot(
                                        "name", {"type": "list", "default": "", "options": ["op1", "op2", "op3"]}
                                    )

                                    assert self._view.annots_order == ["name"]
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QLabel("name")))
                                    QComboBox().setCurrentText.assert_called_once_with("")
                                    QComboBox().addItem.assert_has_calls(
                                        [mock.call("op1"), mock.call("op2"), mock.call("op3")]
                                    )
                                    assert QHBoxLayout().addWidget.has_call(mock.call(QComboBox()))
                                    assert self._view.default_vals == [""]
                                    assert len(self._view.annotation_item_widgets) == 1
                                    assert type(self._view.annotation_item_widgets[0]) == type(QComboBox())
                                    QComboBox().setEnabled.assert_called_once_with(True)
                                    QHBoxLayout().setContentsMargins.assert_called_once_with(2, 12, 8, 12)
                                    QHBoxLayout().setSpacing.assert_called_once_with(2)
                                    QWidget().setLayout.assert_called_once_with(ANY)
                                    assert isinstance(QWidget().setLayout.call_args_list[0][0][0], QHBoxLayout)

                                    QListWidgetItem(self._view.annot_list).setSizeHint.assert_called_once_with(
                                        QWidget().minimumSizeHint()
                                    )

                                    self._view.annot_list.setItemWidget.assert_called_once_with(ANY, ANY)
                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][0], QListWidgetItem
                                    )

                                    assert isinstance(
                                        self._view.annot_list.setItemWidget.call_args_list[0][0][1], QWidget
                                    )

    def test_popup_yes(self):
        with mock.patch.object(QMessageBox, "__init__", lambda x: None):
            QMessageBox.setText = MagicMock()
            QMessageBox.setStandardButtons = MagicMock()
            QMessageBox.exec = MagicMock(return_value=QMessageBox.Yes)

            assert self._view.popup("text")
            QMessageBox.setText.assert_called_once_with("text")
            QMessageBox.setStandardButtons.assert_called_once_with(QMessageBox.No | QMessageBox.Yes)
            QMessageBox.exec.assert_called_once_with()

    def test_popup_no(self):
        with mock.patch.object(QMessageBox, "__init__", lambda x: None):
            QMessageBox.setText = MagicMock()
            QMessageBox.setStandardButtons = MagicMock()
            QMessageBox.exec = MagicMock(return_value=QMessageBox.No)

            assert not self._view.popup("text")
            QMessageBox.setText.assert_called_once_with("text")
            QMessageBox.setStandardButtons.assert_called_once_with(QMessageBox.No | QMessageBox.Yes)
            QMessageBox.exec.assert_called_once_with()



from unittest import mock
from unittest.mock import MagicMock, create_autospec, patch
from napari_allencell_annotator.controller.images_controller import ImagesController
from napari_allencell_annotator.view.images_view import ImagesView, FileItem, ScrollablePopup, QDialog, FilesWidget

from napari_allencell_annotator.view.images_view import AICSImage
from napari_allencell_annotator.view.images_view import napari
from napari_allencell_annotator.view.images_view import exceptions


class TestImagesView:
    def setup_method(self):
        with mock.patch.object(ImagesView, "__init__", lambda x: None):
            self._view = ImagesView()
            self._view.controller: MagicMock = create_autospec(ImagesController)
            self._view.viewer: MagicMock = create_autospec(napari.Viewer)
            self._view.AICSImage = create_autospec(AICSImage)

    def test_update_shuff_text(self):
        # checked
        self._view.shuffle = MagicMock()
        self._view.shuffle.setText = MagicMock()
        self._view._update_shuff_text(True)
        self._view.shuffle.setText.assert_called_once_with("Unhide")
        # not checked
        self._view.shuffle.setText = MagicMock()
        self._view._update_shuff_text(False)
        self._view.shuffle.setText.assert_called_once_with("Shuffle and Hide")

    def test_reset_buttons(self):
        self._view._toggle_delete = MagicMock()
        self._view._toggle_shuffle = MagicMock()
        self._view.shuffle = MagicMock()
        self._view.toggle_add = MagicMock()

        self._view.reset_buttons()
        self._view._toggle_delete.assert_called_once_with(False)
        self._view.shuffle.setChecked.assert_called_once_with(False)
        self._view._toggle_shuffle.assert_called_once_with(False)

        self._view.toggle_add.assert_called_once_with(True)

    def test_delete_clicked(self):
        # test nothing checked
        self._view.file_widget: MagicMock = MagicMock()

        self._view.file_widget.checked = set()
        self._view.alert = MagicMock()
        self._view._delete_clicked()
        self._view.alert.assert_called_once_with("No Images Selected")

    @patch("napari_allencell_annotator.view.images_view.ScrollablePopup.__init__")
    def test_delete_clicked_accept(self, mock_init):
        mock_init.return_value = None
        self._view.file_widget = create_autospec(FilesWidget)
        item = create_autospec(FileItem)
        item.file_path = "path"
        self._view.file_widget.checked = {item}
        self._view.alert = MagicMock()
        ScrollablePopup.exec = MagicMock(return_value=QDialog.Accepted)
        self._view.file_widget.delete_checked = MagicMock()
        self._view._delete_clicked()

        self._view.alert.assert_not_called()

        mock_init.assert_called_once_with("Delete these files from the list?", {"--- path"})
        ScrollablePopup.exec.assert_called_once_with()
        self._view.file_widget.delete_checked.assert_called_once_with()

    @patch("napari_allencell_annotator.view.images_view.ScrollablePopup.__init__")
    def test_delete_clicked_reject(self, mock_init):
        mock_init.return_value = None
        self._view.file_widget: MagicMock = MagicMock()
        item = create_autospec(FileItem)
        item.file_path = "path"
        item2 = create_autospec(FileItem)
        item2.file_path = "path2"
        self._view.file_widget.checked = {item, item2}
        self._view.alert = MagicMock()
        ScrollablePopup.exec = MagicMock(return_value=QDialog.Rejected)
        self._view.file_widget.delete_checked = MagicMock()
        self._view._delete_clicked()

        self._view.alert.assert_not_called()
        mock_init.assert_called_once_with("Delete these files from the list?", {"--- path", "--- path2"})
        ScrollablePopup.exec.assert_called_once_with()
        self._view.file_widget.delete_checked.assert_not_called()

    def test_alert(self):
        with mock.patch("napari_allencell_annotator.view.images_view.show_info") as mock_info:
            self._view.alert("hello")
            mock_info.assert_called_once_with("hello")

    def test_toggle_add(self):
        # check enabled and un-enabled
        self._view.input_dir = MagicMock()
        self._view.input_dir.toggle = MagicMock()
        self._view.input_file = MagicMock()
        self._view.input_file.toggle = MagicMock()
        self._view.toggle_add(True)
        self._view.input_dir.toggle.assert_called_once_with(True)
        self._view.input_file.toggle.assert_called_once_with(True)

        self._view.input_dir.toggle = MagicMock()
        self._view.input_file.toggle = MagicMock()
        self._view.toggle_add(False)
        self._view.input_dir.toggle.assert_called_once_with(False)
        self._view.input_file.toggle.assert_called_once_with(False)

    def test_toggle_delete(self):
        self._view.delete = MagicMock()
        self._view.delete.setEnabled = MagicMock()
        self._view._toggle_delete(True)
        self._view.delete.setEnabled.assert_called_once_with(True)

        self._view.delete.setEnabled = MagicMock()
        self._view._toggle_delete(False)
        self._view.delete.setEnabled.assert_called_once_with(False)

    def test_toggle_shuffle(self):
        self._view.shuffle = MagicMock()
        self._view.shuffle.setEnabled = MagicMock()
        self._view._toggle_shuffle(True)
        self._view.shuffle.setEnabled.assert_called_once_with(True)

        self._view.shuffle.setEnabled = MagicMock()
        self._view._toggle_shuffle(False)
        self._view.shuffle.setEnabled.assert_called_once_with(False)

    def test_display_img_none_both(self):
        # current item none
        self._view.viewer.layers = MagicMock()
        self._view.viewer.layers.clear = MagicMock()
        prev = None
        curr = None
        self._view.AICSImage = MagicMock(return_value="data")
        self._view._display_img(curr, prev)
        self._view.viewer.layers.clear.assert_called_once_with()
        self._view.AICSImage.assert_not_called()

    def test_display_img_none_curr(self):
        # current item none
        self._view.viewer.layers = MagicMock()
        self._view.viewer.layers.clear = MagicMock()
        prev = create_autospec(FileItem)
        prev.unhighlight = MagicMock()
        curr = None
        self._view.AICSImage = MagicMock(return_value="data")
        self._view._display_img(curr, prev)

        self._view.viewer.layers.clear.assert_called_once_with()
        prev.unhighlight.assert_called_once_with()
        self._view.AICSImage.assert_not_called()

    def test_display_img(self):
        self._view.viewer.layers = MagicMock()
        self._view.viewer.layers.clear = MagicMock()
        prev = create_autospec(FileItem)
        prev.unhighlight = MagicMock()
        curr = create_autospec(FileItem)
        curr.file_path = MagicMock(return_value="path")
        curr.highlight = MagicMock()

        self._view.viewer.add_image = MagicMock()

        with mock.patch.object(AICSImage, "__init__", lambda x, y: None):
            AICSImage.data = "data"
            self._view._display_img(curr, prev)

            self._view.viewer.layers.clear.assert_called_once_with()
            self._view.viewer.add_image.assert_called_once_with("data")
            curr.highlight.assert_called_once_with()
            prev.unhighlight.assert_called_once_with()

    def test_display_img_unsupported(self):
        self._view.viewer.layers = MagicMock()
        self._view.viewer.layers.clear = MagicMock()
        prev = create_autospec(FileItem)
        prev.unhighlight = MagicMock()
        curr = create_autospec(FileItem)
        curr.file_path = MagicMock(return_value="path")
        curr.highlight = MagicMock()

        self._view.viewer.add_image = MagicMock()
        with mock.patch.object(exceptions.UnsupportedFileFormatError, "__init__", lambda x, y, z: None):
            AICSImage.__init__ = MagicMock(side_effect=exceptions.UnsupportedFileFormatError("x", "y"))
            AICSImage.data = "data"
            self._view.alert = MagicMock()
            self._view._display_img(curr, prev)

            self._view.viewer.layers.clear.assert_called_once_with()
            self._view.viewer.add_image.assert_not_called()
            curr.highlight.assert_not_called()
            prev.unhighlight.assert_called_once_with()
            self._view.alert.assert_called_once_with("AICS Unsupported File Type")

    def test_display_img_file_not_found(self):
        # current item none
        self._view.viewer.layers = MagicMock()
        self._view.viewer.layers.clear = MagicMock()
        prev = create_autospec(FileItem)
        prev.unhighlight = MagicMock()
        curr = create_autospec(FileItem)
        curr.file_path = MagicMock(return_value="path")
        curr.highlight = MagicMock()
        self._view.file_widget = create_autospec(FilesWidget)

        self._view.viewer.add_image = MagicMock()
        AICSImage.__init__ = MagicMock(side_effect=FileNotFoundError)
        AICSImage.data = "data"
        self._view.alert = MagicMock()
        self._view._display_img(curr, prev)

        self._view.file_widget.remove_item.assert_called_once_with(curr)
        self._view.viewer.layers.clear.assert_called_once_with()
        self._view.viewer.add_image.assert_not_called()
        curr.highlight.assert_not_called()
        prev.unhighlight.assert_called_once_with()
        self._view.alert.assert_called_once_with("File Not Found")



import os
from pathlib import Path
from typing import List, Dict, Optional
import random

import napari

from napari_allencell_annotator.view.images_view import ImagesView
from napari_allencell_annotator.constants.constants import SUPPORTED_FILE_TYPES


class ImagesController:
    """
    A Main controller to integrate view and model for image annotations

    Attributes
    ----------
    view
        the GUI and napari viewer

    Methods
    -------
    load_from_csv(files : List[str], shuffled: bool)
        Adds files to file list from a csv list of file paths.

    get_files_dict() -> Dict[str,List[str]], bool
        Returns the file dictionary that has the current file order and boolean shuffled.

    is_supported(file_path:str)->bool
        Returns True if a file is a supported file type.

    start_annotating(row: Optional[int] = 0)
        Sets the current item to the one at row.

    stop_annotating()
        Clears file widget and reset buttons.

    curr_img_dict() -> Dict[str,str]
        Returns a dictionary with the current image attributes.

    next_img()
        Sets the current image to the next in the list.

    prev_img()
        Sets the current image to the previous one in the list.

    get_num_files(self) -> int
        Returns the number of files.
    """

    def __init__(self, viewer: napari.Viewer):
        self.view: ImagesView = ImagesView(viewer)
        self.view.show()
        self._connect_slots()

    def _connect_slots(self):
        """Connects signals to slots."""
        self.view.input_dir.file_selected.connect(self._dir_selected_evt)
        self.view.input_file.file_selected.connect(self._file_selected_evt)
        self.view.shuffle.clicked.connect(self._shuffle_clicked)

    def load_from_csv(self, files: List[str], shuffled: bool):
        """
        Clear current file list and add file list from csv with its shuffle state.

        If the csv files are shuffled, set shuffled property, hide add button, and hide file names.

        Parameters
        __________
        files : List[str]
            a list of file paths.
        shuffled: bool
            true if files are shuffled.
        """
        self.view.file_widget.clear_all()  # sets shuffled to false
        for name in files:
            self.view.file_widget.add_new_item(name, shuffled)
        if shuffled:
            self.view.file_widget.set_shuffled(True)
            self.view.toggle_add(False)

        self.view.shuffle.setChecked(shuffled)

    def get_files_dict(self) -> (Dict[str, List[str]], bool):
        """
        Return the file dictionary and the shuffle state of file_widget.

        Returns
        ----------
        Dict[str,List[str]], bool
            dictionary of file info. keys in order. a boolean shuffled.
        """
        return self.view.file_widget.files_dict, self.view.file_widget.shuffled

    def _shuffle_clicked(self, checked: bool):
        """
        Shuffle file order and hide file names if checked.
        Return files to original order and names if unchecked.

        Side effect: set file_widget.shuffled_files_dict to a new order dict or {} if list is unshuffled.

        Parameters
        ----------
        checked : bool
            Toggle state of the shuffle button.
        """
        if checked:
            files: Dict[str, List[str]] = self.view.file_widget.clear_for_shuff()
            if len(files) > 0:
                self.view.toggle_add(False)
                keys = list(files.keys())
                self.view.file_widget.files_dict = {}
                random.shuffle(keys)
                for k in keys:
                    # add new item will recreate files_dict in new order
                    self.view.file_widget.add_new_item(k, hidden=True)

        else:
            self.view.toggle_add(True)
            self.view.file_widget.set_shuffled(False)
            self.view.file_widget.unhide_all()

    @staticmethod
    def is_supported(file_path: str) -> bool:
        """
        Check if the provided file name is a supported file.

        This function checks if the file name extension is in
        the supported file types files.

        Parameters
        ----------
        file_path : str
            Name of the file to check.

        Returns
        -------
        bool
            True if the file is supported.
        """
        if file_path is None:
            return False
        extension = Path(file_path).suffix
        if extension in SUPPORTED_FILE_TYPES:
            return True
        else:
            return False

    def _dir_selected_evt(self, dir_list: List[str]):
        """
        Adds all files in a directory to the GUI.

        Parameters
        ----------
        dir_list : List[str]
            The input list with dir[0] holding directory name.
        """
        if dir_list is not None and len(dir_list) > 0:
            d = dir_list[0]
            if len(os.listdir(d)) < 1:
                self.view.alert("Folder is empty")
            else:
                for file in [file for file in os.listdir(d) if not file.startswith(".")]:

                    file = d + "/" + file
                    if self.is_supported(file):
                        self.view.file_widget.add_new_item(file)
                    else:
                        self.view.alert("Unsupported file type(s)")
        else:
            self.view.alert("No selection provided")

    def _file_selected_evt(self, file_list: List[str]):
        """
        Adds all selected files to the GUI.

        Parameters
        ----------
        file_list : List[str]
            The list of files
        """
        if file_list is None or len(file_list) < 1:
            self.view.alert("No selection provided")
        else:
            for file in file_list:
                if self.is_supported(file):
                    self.view.file_widget.add_new_item(file)
                else:
                    self.view.alert("Unsupported file type(s)")

    def start_annotating(self, row: Optional[int] = 0):
        """Set current item to the one at row."""
        if self.view.file_widget.count() > 0:
            self.view.file_widget.setCurrentItem(self.view.file_widget.item(row))

        else:
            self.view.alert("No files to annotate")

    def stop_annotating(self):
        """Clear file widget and reset buttons."""
        self.view.file_widget.clear_all()
        self.view.reset_buttons()

    def curr_img_dict(self) -> Dict[str, str]:
        """
         Return a dictionary with the current image File Path
         and row in the list.

        Returns
        ----------
        Dict[str,str]
            dictionary of file name and row attributes.
        """
        item = self.view.file_widget.currentItem()
        info = {
            "File Path": item.file_path,
            "Row": str(self.view.file_widget.get_curr_row()),
        }
        return info

    def next_img(self):
        """
        Set the current image to the next in the list, stop incrementing
        at the last row.
        """
        if self.view.file_widget.get_curr_row() < self.view.file_widget.count() - 1:
            self.view.file_widget.setCurrentItem(self.view.file_widget.item(self.view.file_widget.get_curr_row() + 1))

    def prev_img(self):
        """Set the current image to the previous in the list, stop at first image."""
        if self.view.file_widget.get_curr_row() > 0:
            self.view.file_widget.setCurrentItem(self.view.file_widget.item(self.view.file_widget.get_curr_row() - 1))

    def get_num_files(self) -> int:
        """
        Returns
        ----------
        int
            number of files.
        """
        return self.view.file_widget.count()
